@@ -27,12 +27,13 @@ use LaTeXML::Common::Font;
 # Silly place to import these....?
 use LaTeXML::Common::Color;
 use LaTeXML::Core::Definition;
+use Scalar::Util qw(blessed);
 use base qw(LaTeXML::Common::Object);
 
 #**********************************************************************
 sub new {
   my ($class, %options) = @_;
-  return bless { gullet => LaTeXML::Core::Gullet->new(),
+  return bless { gullet => LaTeXML::Core::Gullet->new(%options),
     boxing => [], token_stack => [] }, $class; }
 
 #**********************************************************************
@@ -41,6 +42,7 @@ sub initialize {
   my ($self) = @_;
   $$self{boxing}      = [];
   $$self{token_stack} = [];
+  $$self{progress}    = 0;
   $STATE->assignValue(MODE              => 'text',           'global');
   $STATE->assignValue(IN_MATH           => 0,                'global');
   $STATE->assignValue(PRESERVE_NEWLINES => 1,                'global');
@@ -52,6 +54,8 @@ sub initialize {
   $STATE->assignValue(mathfont => LaTeXML::Common::Font->mathDefault(), 'global');
   return; }
 
+our $DIGESTION_PROGRESS_QUANTUM = 30000;
+
 #**********************************************************************
 sub getGullet {
   my ($self) = @_;
@@ -65,6 +69,17 @@ sub getBoxingLevel {
   my ($self) = @_;
   return scalar(@{ $$self{boxing} }); }
 
+# ScriptLevel is similar to boxing level, but relative to current Math mode's level
+# This is used for the scriptpos attribute to recognize overlapping sccripts.
+# Making it relative to the math's level avoids unnecessary changes
+sub getScriptLevel {
+  my ($self) = @_;
+  my $boxlevel = scalar(@{ $$self{boxing} });
+  if (my $prevlevel = $STATE->lookupValue('script_base_level')) {
+    return $boxlevel - $prevlevel + 1; }
+  else {
+    return $boxlevel; } }
+
 #**********************************************************************
 # Digestion
 #**********************************************************************
@@ -78,12 +93,18 @@ sub digestNextBody {
   my $initdepth = scalar(@{ $$self{boxing} });
   my $token;
   local @LaTeXML::LIST = ();
+  my $alignment = $STATE->lookupValue('Alignment');
+  my @aug       = ();
+
   while (defined($token = $$self{gullet}->readXToken(1, 1))) {    # Done if we run out of tokens
-    push(@LaTeXML::LIST, $self->invokeToken($token));
+    my @r = $self->invokeToken($token);
+    push(@LaTeXML::LIST, @r);
+    push(@aug, $token, @r);
     last if $terminal and Equals($token, $terminal);
     last if $initdepth > scalar(@{ $$self{boxing} }); }           # if we've closed the initial mode.
   Warn('expected', $terminal, $self, "body should have ended with '" . ToString($terminal) . "'",
-    "current body started at " . ToString($startloc))
+    "current body started at " . ToString($startloc),
+    "Got " . join("\n -- ", map { Stringify($_) } @aug))
     if $terminal && !Equals($token, $terminal);
   push(@LaTeXML::LIST, Box()) unless $token;                      # Dummy `trailer' if none explicit.
   return @LaTeXML::LIST; }
@@ -99,23 +120,15 @@ sub digest {
     $$self{gullet}->readingFromMouth(LaTeXML::Core::Mouth->new(), sub {
       my ($gullet) = @_;
       $gullet->unread($tokens);
-      $STATE->clearPrefixes;    # prefixes shouldn't apply here.
+      $STATE->clearPrefixes;                                             # prefixes shouldn't apply here.
       my $ismath    = $STATE->lookupValue('IN_MATH');
       my $initdepth = scalar(@{ $$self{boxing} });
-      my $depth     = $initdepth;
       local @LaTeXML::LIST = ();
       while (defined(my $token = $$self{gullet}->readXToken(1, 1))) {    # Done if we run out of tokens
         push(@LaTeXML::LIST, $self->invokeToken($token));
         last if $initdepth > scalar(@{ $$self{boxing} }); }              # if we've closed the initial mode.
-      Fatal('internal', '<EOF>', $self,
-        "We've fallen off the end, somehow!?!?!",
-        "Last token " . ToString($LaTeXML::CURRENT_TOKEN)
-          . " (Boxing depth was $initdepth, now $depth: Boxing generated by "
-          . join(', ', map { ToString($_) } @{ $$self{boxing} }))
-        if $initdepth < $depth;
-
       List(@LaTeXML::LIST, mode => ($ismath ? 'math' : 'text'));
-      }); }
+    }); }
 
 # Invoke a token;
 # If it is a primitive or constructor, the definition will be invoked,
@@ -125,13 +138,18 @@ sub digest {
 my $MAXSTACK = 200;    # [CONSTANT]
 
 # Overly complex, but want to avoid recursion/stack
-my @absorbable_cc = (    # [CONSTANT]
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 0);
+our @CATCODE_ABSORBABLE = (    # [CONSTANT]
+  0, 0, 0, 0,
+  0, 0, 0, 0,
+  0, 0, 1, 1,
+  1, 0, 1, 0,
+  0, 0, 0, 0);
 
 sub invokeToken {
   my ($self, $token) = @_;
   no warnings 'recursion';
 INVOKE:
+  ProgressStep() if ($$self{progress}++ % $DIGESTION_PROGRESS_QUANTUM) == 0;
   push(@{ $$self{token_stack} }, $token);
   if (scalar(@{ $$self{token_stack} }) > $MAXSTACK) {
     Fatal('internal', '<recursion>', $self,
@@ -141,13 +159,22 @@ INVOKE:
   my @result  = ();
   my $meaning = $STATE->lookupDigestableDefinition($token);
 
-  if ($meaning->isaToken) {    # Common case
+  if (!$meaning) {
+    @result = $self->invokeToken_undefined($token); }
+  elsif ($meaning->isaToken) {    # Common case
     my $cc = $meaning->getCatcode;
     if ($cc == CC_CS) {
       @result = $self->invokeToken_undefined($token); }
-    elsif ($absorbable_cc[$cc]) {
+    elsif ($CATCODE_ABSORBABLE[$cc]) {
       @result = $self->invokeToken_simple($token, $meaning); }
     else {
+      # Special error guard for the align char "&":
+      # Locally deactivate to avoid a flurry of errors in the same table.
+      # Alert the end user once per table, allowing longer documents to not
+      # hit 100 errors too quickly.
+      $STATE->assignMeaning($token,
+        $STATE->lookupMeaning(T_CS('\relax')), 'local')
+        if Equals($token, T_ALIGN);
       Error('misdefined', $token, $self,
         "The token " . Stringify($token) . " should never reach Stomach!");
       @result = $self->invokeToken_simple($token, $meaning); } }
@@ -155,81 +182,58 @@ INVOKE:
   # but it isn't expanded in the gullet, but later when digesting, in math mode (? I think)
   elsif ($meaning->isExpandable) {
     my $gullet = $$self{gullet};
-    $gullet->unread(@{ $meaning->invoke($gullet) || [] });
+    $gullet->unread($meaning->invoke($gullet));
     $token = $gullet->readXToken();    # replace the token by it's expansion!!!
     pop(@{ $$self{token_stack} });
-    goto INVOKE; }
+    goto INVOKE if $token; }
   elsif ($meaning->isaDefinition) {    # Otherwise, a normal primitive or constructor
     @result = $meaning->invoke($self);
     $STATE->clearPrefixes unless $meaning->isPrefix; }    # Clear prefixes unless we just set one.
   else {
-    Fatal('misdefined', $meaning, $self,
-      "The object " . Stringify($meaning) . " should never reach Stomach!"); }
+    Error('misdefined', $meaning, $self,
+      "The object " . Stringify($meaning) . " should never reach Stomach!");
+    @result = (makeMisdefinedError($meaning)); }
   if ((scalar(@result) == 1) && (!defined $result[0])) {
     @result = (); }                                       # Just paper over the obvious thing.
-  Fatal('misdefined', $token, $self,
-    "Execution yielded non boxes",
-    "Returned " . join(',', map { "'" . Stringify($_) . "'" }
-        grep { (!ref $_) || (!$_->isaBox) } @result))
-    if grep { (!ref $_) || (!$_->isaBox) } @result;
-
+  if (my @bad = grep { (!blessed $_) || (!$_->isaBox) } @result) {
+    Error('misdefined', $token, $self,
+      "Execution yielded non boxes",
+      "Returned " . join(',', map { "'" . Stringify($_) . "'" } @bad));
+    @result = (makeMisdefinedError(@result)); }
   pop(@{ $$self{token_stack} });
   return @result; }
 
-sub makeError {
-  my ($document, $type, $content) = @_;
-  my $savenode = undef;
-  $savenode = $document->floatToElement('ltx:ERROR')
-    unless $document->isOpenable('ltx:ERROR');
-  $document->openElement('ltx:ERROR', class => ToString($type));
-  $document->openText_internal(ToString($content));
-  $document->closeElement('ltx:ERROR');
-  $document->setNode($savenode) if $savenode;
-  return; }
+sub makeMisdefinedError {
+  my (@objects) = @_;
+  return LaTeXML::Core::Whatsit->new($STATE->lookupDefinition(T_CS('\@ERROR')),
+    ['misdefined', join('', map { ToString($_); } @objects)],
+    font => $STATE->lookupValue('font'),
+  ); }
 
 sub invokeToken_undefined {
   my ($self, $token) = @_;
-  my $cs = $token->getCSName;
-  $STATE->noteStatus(undefined => $cs);
-  # To minimize chatter, go ahead and define it...
-  if ($cs =~ /^\\if(.*)$/) {    # Apparently an \ifsomething ???
-    my $name = $1;
-    Error('undefined', $token, $self, "The token " . Stringify($token) . " is not defined.",
-      "Defining it now as with \\newif");
-    $STATE->installDefinition(LaTeXML::Core::Definition::Expandable->new(
-        T_CS('\\' . $name . 'true'), undef, '\let' . $cs . '\iftrue'));
-    $STATE->installDefinition(LaTeXML::Core::Definition::Expandable->new(
-        T_CS('\\' . $name . 'false'), undef, '\let' . $cs . '\iffalse'));
-    LaTeXML::Package::Let($token, T_CS('\iffalse'));
-    $self->getGullet->unread($token);    # Retry
-    return; }
-  else {
-    Error('undefined', $token, $self, "The token " . Stringify($token) . " is not defined.",
-      "Defining it now as <ltx:ERROR/>");
-    $STATE->installDefinition(LaTeXML::Core::Definition::Constructor->new($token, undef,
-        sub { makeError($_[0], 'undefined', $cs); }),
-      'global');
-    # and then invoke it.
-    return $self->invokeToken($token); } }
+  $STATE->generateErrorStub($self, $token);
+  $self->getGullet->unread($token);    # Retry
+  return; }
 
 sub invokeToken_simple {
   my ($self, $token, $meaning) = @_;
   my $cc   = $meaning->getCatcode;
   my $font = $STATE->lookupValue('font');
-  $STATE->clearPrefixes;    # prefixes shouldn't apply here.
+  $STATE->clearPrefixes;               # prefixes shouldn't apply here.
   if ($cc == CC_SPACE) {
     if ($STATE->lookupValue('IN_MATH')) {    # (but in Preamble, OK ?)
       return (); }
     else {
-      return Box($meaning->getString, $font, $$self{gullet}->getLocator, $meaning); } }
+      return Box($meaning->toString, $font, $self->getGullet->getLocator, $meaning); } }
   elsif ($cc == CC_COMMENT) {                # Note: Comments need char decoding as well!
-    my $comment = LaTeXML::Package::FontDecodeString($meaning->getString, undef, 1);
+    my $comment = LaTeXML::Package::FontDecodeString($meaning->toString, undef, 1);
     # However, spaces normally would have be digested away as positioning...
     my $badspace = pack('U', 0xA0) . "\x{0335}";    # This is at space's pos in OT1
     $comment =~ s/\Q$badspace\E/ /g;
     return LaTeXML::Core::Comment->new($comment); }
   else {
-    return Box(LaTeXML::Package::FontDecodeString($meaning->getString, undef, 1),
+    return Box(LaTeXML::Package::FontDecodeString($meaning->toString, undef, 1),
       undef, undef, $meaning); } }
 
 # Regurgitate: steal the previously digested boxes from the current level.
@@ -266,7 +270,8 @@ sub popStackFrame {
     if (@$beforeafter) {
       my @result = map { $_->beDigested($self) } @$beforeafter;
       if (my ($x) = grep { !$_->isaBox } @result) {
-        Fatal('misdefined', $x, $self, "Expected a Box|List|Whatsit, but got '" . Stringify($x) . "'"); }
+        Error('misdefined', $x, $self, "Expected a Box|List|Whatsit, but got '" . Stringify($x) . "'");
+        @result = (makeMisdefinedError(@result)); }
       push(@LaTeXML::LIST, @result); } }
   my $after = $STATE->lookupValue('afterGroup');
   $STATE->popFrame;
@@ -298,6 +303,11 @@ sub currentFrameMessage {
 sub bgroup {
   my ($self) = @_;
   pushStackFrame($self, 0);
+  # NOTE: This is WRONG; should really only track "scanned" (not digested) braces
+  # Alas, there're too many code structuring differences between TeX and LaTeXML
+  # to find all the places to manage it.
+  # So, let's try this for now...
+  $LaTeXML::ALIGN_STATE++;
   return; }
 
 sub egroup {
@@ -308,6 +318,7 @@ sub egroup {
       $self->currentFrameMessage); }
   else {                                        # Don't pop if there's an error; maybe we'll recover?
     popStackFrame($self, 0); }
+  $LaTeXML::ALIGN_STATE--;
   return; }
 
 sub begingroup {
@@ -339,21 +350,22 @@ sub setMode {
   $STATE->assignValue(MODE    => $mode,   'local');
   $STATE->assignValue(IN_MATH => $ismath, 'local');
   my $curfont = $STATE->lookupValue('font');
-  if ($mode eq $prevmode) { }
+  if    ($mode eq $prevmode) { }
   elsif ($ismath) {
     # When entering math mode, we set the font to the default math font,
     # and save the text font for any embedded text.
-    $STATE->assignValue(savedfont => $curfont, 'local');
+    $STATE->assignValue(savedfont         => $curfont, 'local');
+    $STATE->assignValue(script_base_level => scalar(@{ $$self{boxing} }));    # See getScriptLevel
     $STATE->assignValue(font => $STATE->lookupValue('mathfont')->merge(
-        color => $curfont->getColor, background => $curfont->getBackground,
-        size => $curfont->getSize,
+        color     => $curfont->getColor, background => $curfont->getBackground,
+        size      => $curfont->getSize,
         mathstyle => ($mode =~ /^display/ ? 'display' : 'text')), 'local'); }
   else {
     # When entering text mode, we should set the font to the text font in use before the math
     # but inherit color and size
     $STATE->assignValue(font => $STATE->lookupValue('savedfont')->merge(
         color => $curfont->getColor, background => $curfont->getBackground,
-        size => $curfont->getSize), 'local'); }
+        size  => $curfont->getSize), 'local'); }
   return; }
 
 sub beginMode {
@@ -377,7 +389,7 @@ sub endMode {
 
 __END__
 
-=pod 
+=pod
 
 =head1 NAME
 
@@ -412,7 +424,7 @@ are collected into a L<LaTeXML::Core::List>.
 
 =item Constructors
 
-A special class of control sequence, called a L<LaTeXML::Core::Definition::Constructor> produces a 
+A special class of control sequence, called a L<LaTeXML::Core::Definition::Constructor> produces a
 L<LaTeXML::Core::Whatsit> which remembers the control sequence and arguments that
 created it, and defines its own translation into C<XML> elements, attributes and data.
 Arguments to a constructor are read from the gullet and also digested.
@@ -427,7 +439,7 @@ Arguments to a constructor are read from the gullet and also digested.
 
 Return the digested L<LaTeXML::Core::List> after reading and digesting a `body'
 from the its Gullet.  The body extends until the current
-level of boxing or environment is closed.  
+level of boxing or environment is closed.
 
 =item C<< $list = $stomach->digest($tokens); >>
 
@@ -445,7 +457,7 @@ A List of Box's, Lists, Whatsit's is returned.
 
 =item C<< @boxes = $stomach->regurgitate; >>
 
-Removes and returns a list of the boxes already digested 
+Removes and returns a list of the boxes already digested
 at the current level.  This peculiar beast is used
 by things like \choose (which is a Primitive in TeX, but
 a Constructor in LaTeXML).
