@@ -54,7 +54,7 @@ sub new {
   # Parameters for separating the clipping box from the
   # desired padding between image edge and "ink"
   $$self{padding} = $options{padding} || 2;    # pixels
-        # amount of extra space (+padding) to put between object & rules for clipping
+      # amount of extra space (+padding) to put between object & rules for clipping
   $$self{clippingfudge} = 3;       # px
   $$self{clippingrule}  = 0.90;    # pixels (< 1 to avoid antialiasing..?)
 
@@ -81,19 +81,14 @@ sub new {
   #  my $fmt = ($^O eq 'MSWin32' ? '%%' : '%');
   my $fmt = '%';
   if ($$self{use_dvisvgm}) {
-    # img name uses 2 digits for page; what happens at 100?
     ##    $$self{dvicmd}             = "dvisvgm --page=1- --bbox=min --mag=$mag -o imgx-${fmt}p";
     # dvisvgm currently creates glyph descriptions w/ unicode attribute having the wrong codepoint
     # firefox, chromium use this codepoint instead of the glyph "drawing"
     # a later version of dvisvgm should do better at synthesizing the unicode?
     # but for now, we'll use --no-fonts, which creates glyph drawings rather than "glyphs"
     # Also, increase the bounding box from min by 1pt
-    # Also, note incompatible change in the -o option after version 1.6;
-    # it now can take a number of digits (%3p) for pages, BUT old version produces imgx-%3p.svg!!
-    # So, simply use %p, let new version do 1,..,9,10,...
-    # Recovery code below!
-    $$self{dvicmd} = "dvisvgm --page=1- --bbox=1pt --scale=$$self{magnification} --no-fonts -o imgx-${fmt}p";
-    $$self{dvicmd_output_name} = 'imgx-%02d.svg';
+    $$self{dvicmd} = "dvisvgm --page=1- --bbox=1pt --scale=$$self{magnification} --no-fonts -o imgx-${fmt}3p";
+    $$self{dvicmd_output_name} = 'imgx-%03d.svg';
     $$self{dvicmd_output_type} = 'svg';
     $$self{frame_output}       = 0; }
   elsif ($$self{use_dvipng}) {
@@ -181,7 +176,7 @@ sub cleanTeX {
   if ($tex =~ s/^\s*(\\displaystyle|\\textstyle|\\scriptstyle|\\scriptscriptstyle)\s*//) {
     $style = $1; }
   $tex =~ s/^(?:\\\s*,|\\!\s*|\\>\s*|\\;\s*|\\:\s*|\\ \s*|\\\/\s*)*//; # Trim leading spacing (especially negative!)
-  $tex =~ s/(?:\\\s*,|\\!\s*|\\>\s*|\\;\s*|\\:\s*|\\ \s*|\\\/\s*)*$//; # and trailing spacing
+  $tex =~ s/(?:\\\s*,|\\!\s*|\\>\s*|\\;\s*|\\:\s*|\\ \s*|\\\/\s*)*$//;    # and trailing spacing
       # Strip comments, but watchout for \% (or more exactly, an ODD number of \)
   $tex =~ s/(?<!\\)((?:\\\\)*)\%[^\n]*\n/$1/gs;
   $tex = $style . ' ' . $tex if $style;    # Put back the style, if any
@@ -253,7 +248,7 @@ sub generateImages {
       next if -f pathname_absolute($1, $destdir); }
     push(@pending, $table{$key}); }
 
-  NoteProgress(" [$nuniq unique; " . scalar(@pending) . " new]");
+  Debug("LaTeXImages: $nuniq unique; " . scalar(@pending) . " new") if $LaTeXML::DEBUG{images};
   if (@pending) {    # if any images need processing
         # Create working directory; note TMPDIR attempts to put it in standard place (like /tmp/)
     File::Temp->safe_level(File::Temp::HIGH);
@@ -286,14 +281,15 @@ sub generateImages {
     # Sometimes latex returns non-zero code, even though it apparently succeeded.
     # And sometimes it doesn't produce a dvi, even with 0 return code?
     if (($ltxerr != 0) || (!-f "$workdir/$jobname.dvi")) {
-      my $preserve = $$LaTeXML::POST{verbosity} > 0;    # preserve junk when verbosity high.
-      $workdir->unlink_on_destroy(0) if $preserve;
+      $workdir->unlink_on_destroy(0) if $LaTeXML::DEBUG{images};    # Preserve junk
       Error('shell', $LATEXCMD, undef,
         "LaTeX command '$ltxcommand' failed",
-        ($ltxerr == 0 ? "No dvi file generated"     : "returned code $ltxerr (!= 0): $@"),
-        ($preserve    ? "See $workdir/$jobname.log" : "Re-run with --verbose to see TeX log"));
+        ($ltxerr == 0 ? "No dvi file generated" : "returned code $ltxerr (!= 0): $@"),
+        ($LaTeXML::DEBUG{images}
+          ? "See $workdir/$jobname.log"
+          : "Re-run with --debug=images to see TeX log"));
       return $doc; }
-    $workdir->unlink_on_destroy(0) if $$LaTeXML::POST{verbosity} > 2;
+    ### $workdir->unlink_on_destroy(0) if $LaTeXML::DEBUG{images}; # preserve ALL junk!?!?!
 
     # Extract dimensions (width x height+depth) from each image from log file.
     my @dimensions = ();
@@ -324,10 +320,6 @@ sub generateImages {
     my ($index, $ndigits) = (0, 1 + int(log($doc->cacheLookup((ref $self) . ':_max_image_') || 1) / log(10)));
     foreach my $entry (@pending) {
       my $src = "$workdir/" . sprintf($$self{dvicmd_output_name}, ++$index);
-      # Recovery patchup for incompatible change to dvisvgm!
-      if (($index == 1) && (!-f $src) && (-f "$workdir/imgx-1.svg")) {
-        $$self{dvicmd_output_name} = 'imgx-%d.svg';
-        $src = "$workdir/imgx-1.svg"; }
       if (-f $src) {
         my @dests = @{ $$entry{dest} };
         push(@dests, $self->generateResourcePathname($doc, $$entry{nodes}[0], undef, $$self{imagetype}))
@@ -345,7 +337,6 @@ sub generateImages {
           my $d = int(0.5 + ($dd || 0) + $$self{padding});
           if ((($w == 1) && ($ww > 1)) || (($h == 1) && ($hh > 1))) {
             Warn('expected', 'image', undef, "Image for '$$entry{tex}' was cropped to nothing!"); }
-#          print STDERR "\nFor key='$$entry{key}': Image[$index] '$dest' $$entry{tex} $ww x $hh + $dd ==> $w x $h \\ $d\n";
           $doc->cacheStore($$entry{key}, "$dest;$w;$h;$d"); }
       }
       else {
@@ -378,6 +369,7 @@ sub pre_preamble {
   my $dest            = $doc->getDestination;
   my $description     = ($dest ? "% Destination $dest" : "");
   my $pts_per_pixel   = 72.27 / $$self{dpi} / $$self{magnification};
+
   foreach my $pkgdata (@classdata) {
     my ($package, $package_options) = @$pkgdata;
     $package_options = "[$package_options]" if $package_options && ($package_options !~ /^\[.*\]$/);
@@ -388,7 +380,8 @@ sub pre_preamble {
   my $th  = ($$self{frame_output}
     ? $$self{clippingrule} * $pts_per_pixel    # clipping box thickness in points.
     : 0);                                      # NO clipping box!
-                                               #  print STDERR "w=$w, gap=$gap, thickness=$th\n";
+  my $preambles = $self->find_preambles($doc);
+
   return <<"EOPreamble";
 \\batchmode
 \\def\\inlatexml{true}
@@ -399,10 +392,6 @@ $packages
 \\setlength{\\hoffset}{0pt}\\setlength{\\voffset}{0pt}
 \\setlength{\\textwidth}{${w}pt}
 \\thispagestyle{empty}\\pagestyle{empty}\\title{}\\author{}\\date{}
-% Extra definitions for LaTeXML generated TeX
-\\def\\FCN#1{#1}
-\\def\\DUAL{\\\@ifnextchar[{\\\@DUAL}{\\\@DUAL[]}}
-\\def\\\@DUAL[#1]#2#3{#3}% Use presentation form!!!
 \\newcount\\lxImageNumber\\lxImageNumber=0\\relax
 \\newbox\\lxImageBox
 \\newdimen\\lxImageBoxSep
@@ -438,6 +427,7 @@ $packages
 }%
 \\def\\lxBeginImage{\\setbox\\lxImageBox\\hbox\\bgroup\\color\@begingroup\\kern\\lxImageBoxSep}
 \\def\\lxEndImage{\\kern\\lxImageBoxSep\\color\@endgroup\\egroup}
+$preambles
 \\makeatother
 EOPreamble
 }
@@ -457,7 +447,7 @@ sub convert_image {
   my ($bg, $fg) = ($$self{background}, 'black');
 
   my $image = image_object(antialias => 'True', background => $bg, density => $$self{dpi});
-  my $err = $image->Read($$self{dvicmd_output_type} . ':' . $src);
+  my $err   = $image->Read($$self{dvicmd_output_type} . ':' . $src);
   if ($err) {
     Warn('imageprocessing', 'read', undef,
       "Image conversion failed to read '$src'",
@@ -471,16 +461,16 @@ sub convert_image {
   $image->Border(width => 1, height => 1, fill => $bg);
   $image->Trim(fuzz => '75%');    # Fuzzy, to trim the gray tabs, as well!!
   $image->Set(fuzz => '0%');      # But, be SURE to RESET fuzz!! (It is NOT an "argument"!!!)
-       # [Also, be CAREFULL of ImageMagick's Floodfill variants, they sometimes go wild]
+      # [Also, be CAREFULL of ImageMagick's Floodfill variants, they sometimes go wild]
 
   # Finally, shave off the rule & fudge padding.
   my $fudge = int(0.5 + $$self{clippingfudge} + $$self{clippingrule});
   $image->Shave(width => $fudge, height => $fudge);
 
   my ($w, $h) = $image->Get('width', 'height');    # Get final image size
-         # ImageMagick tries to manage a "virtual" image within the image data,
-         # (whatever that means)
-         # This resets it back to the origin which avoids confusion, all 'round!
+      # ImageMagick tries to manage a "virtual" image within the image data,
+      # (whatever that means)
+      # This resets it back to the origin which avoids confusion, all 'round!
   $image->Set(page => "${w}x${h}+0+0");
 
   # Ideally, we'd make the alpha exactly opposite the ink, rather than merely 1 bit
@@ -488,8 +478,7 @@ sub convert_image {
   #  $image = $image->Fx(expression=>'(3.0-r+g+b)/3.0', channel=>'alpha');
   $image->Transparent(color => $bg);
 
-  NoteProgressDetailed(" [Converting $src => $dest ($w x $h)]");
-
+  Debug("LaTeXImages: Converting $src => $dest ($w x $h)") if $LaTeXML::DEBUG{images};
   # Workaround bad png detection(?) in ImageMagick 6.6.5 (???)
   if ($$self{imagetype} eq 'png') {
     $dest = "png32:$dest"; }
@@ -503,7 +492,7 @@ sub DESTROY {
       my @empty_magick = grep { -z $_ } map { "$tmpdir/$_" } readdir($tmpdir_fh);
       closedir($tmpdir_fh);
       unlink $_ foreach @empty_magick;
-    } }
+  } }
   return; }
 
 #======================================================================
