@@ -38,12 +38,32 @@ our $DPI            = 90;               # [CONSTANT]
 our $DOTS_PER_POINT = ($DPI / 72.0);    # [CONSTANT] Dots per point.
 our $BACKGROUND     = "#FFFFFF";        # [CONSTANT]
 
-# Note that Image::Size my, itself, use Image::Magick, if available,
+# These environment variables can be used to limit the amount
+# of time & space used by ImageMagick.  They are particularly useful
+# when ghostscript becomes involved in converting postscript or pdf.
+# HOWEVER, there are indications that the time limit (at least)
+# is being measured against the whole latexml process, not just image processing.
+# Thus they aren't really that useful here.
+# They probably are useful in a server context, however, so I'll leave the comments.
+# $ENV{MAGICK_DISK_LIMIT} = "2GiB" unless defined $ENV{MAGICK_DISK_LIMIT};
+# $ENV{MAGICK_MEMORY_LIMIT} = "512MiB" unless defined $ENV{MAGICK_MEMORY_LIMIT};
+# $ENV{MAGICK_MAP_LIMIT} = "1GiB" unless defined $ENV{MAGICK_MAP_LIMIT};
+# $ENV{MAGICK_TIME_LIMIT} = "300" unless defined $ENV{MAGICK_TIME_LIMIT};
+
+# Note that Image::Size may, itself, use Image::Magick, if available,
 # as a fallback for getting image size & type!!!
+# However, it seems not to recognize file types with extension .EPS (uppercase), eg!
 sub image_type {
   my ($pathname) = @_;
   my ($w, $h, $t) = imgsize($pathname);
-  return lc($t); }
+  # even though Image::Size CLAIMS to use Image::Magick as fallback... needs tickling?
+  if (!(defined $w) && !(defined $h) && image_can_image()) {    # try harder!
+    my $image = image_read($pathname) or return;
+    ($t) = image_getvalue($image, 'format'); }
+  # Note that Image::Magick (sometimes) returns "descriptive" types
+  # (but I can't find a list anywhere!)
+  $t = 'eps' if $t && $t =~ /postscript/i;
+  return (defined $t ? lc($t) : undef); }
 
 sub image_size {
   my ($pathname) = @_;
@@ -66,7 +86,9 @@ sub image_can_image {
     foreach my $class (@MagickClasses) {
       my $module = $class . ".pm";
       $module =~ s/::/\//g;
-      my $object = eval { local $SIG{__DIE__} = undef; require $module; $class->new(); };
+      my $object = eval {
+        local $LaTeXML::IGNORE_ERRORS = 1;
+        require $module; $class->new(); };
       if ($object) {
         $IMAGECLASS = $class;
         last; } } }
@@ -101,34 +123,34 @@ sub image_graphicx_parse {
   my ($v, $clip, $trim, $width, $height, $xscale, $yscale,
     $aspect, $angle, $rotfirst, $mag, @bb, @vp,) = ('', '', '', 0, 0, 0, 0, '', 0, '', 1, 0);
   my @unknown = ();
-  my @ignore = @{ $options{ignore_options} || [] };
+  my @ignore  = @{ $options{ignore_options} || [] };
   foreach (split(',', $transformstring || '')) {
     if (/^\s*(\w+)(?:=\s*(.*))?\s*$/) {
       $_ = $1; $v = $2 || '';
       my $op = $_;
-      if (grep { $op eq $_ } @ignore) { }    # Ignore this option
-      elsif (/^bb$/) { @bb = map { to_bp($_) } split(' ', $v); }
-      elsif (/^bb(?:ll|ur)(?:x|y)$/) { $bb[2 * /ur/ + /y/] = to_bp($v); }
+      if (grep { $op eq $_ } @ignore) { }                                           # Ignore this option
+      elsif (/^bb$/)                  { @bb = map { to_bp($_) } split(' ', $v); }
+      elsif (/^bb(?:ll|ur)(?:x|y)$/)  { $bb[2 * /ur/ + /y/] = to_bp($v); }
       elsif (/^nat(?:width|height)$/) { $bb[2 + /width/] = to_bp($v); }
-      elsif (/^viewport$/) { @vp = map { to_bp($_) } split(' ', $v); $trim = 0; }
-      elsif (/^trim$/)     { @vp = map { to_bp($_) } split(' ', $v); $trim = 1; }
-      elsif (/^clip$/)             { $clip   = !($v eq 'false'); }
-      elsif (/^keepaspectratio$/)  { $aspect = !($v eq 'false'); }
-      elsif (/^width$/)            { $width  = to_bp($v); }
-      elsif (/^(?:total)?height$/) { $height = to_bp($v); }
-      elsif (/^scale$/)            { $xscale = $yscale = $v; }
-      elsif (/^xscale$/)           { $xscale = $v; }
-      elsif (/^yscale$/)           { $yscale = $v; }
-      elsif (/^angle$/) { $angle = $v; $rotfirst = !($width || $height || $xscale || $yscale); }
-      elsif (/^origin$/)        { }              # ??
-                                                 # Non-standard option
+      elsif (/^viewport$/)            { @vp = map { to_bp($_) } split(' ', $v); $trim = 0; }
+      elsif (/^trim$/)                { @vp = map { to_bp($_) } split(' ', $v); $trim = 1; }
+      elsif (/^clip$/)                { $clip = !($v eq 'false'); }
+      elsif (/^keepaspectratio$/)     { $aspect = !($v eq 'false'); }
+      elsif (/^width$/)               { $width = to_bp($v); }
+      elsif (/^(?:total)?height$/)    { $height = to_bp($v); }
+      elsif (/^scale$/)               { $xscale = $yscale = $v; }
+      elsif (/^xscale$/)              { $xscale = $v; }
+      elsif (/^yscale$/)              { $yscale = $v; }
+      elsif (/^angle$/)  { $angle = $v; $rotfirst = !($width || $height || $xscale || $yscale); }
+      elsif (/^origin$/) { }                                                                        # ??
+          # Non-standard option
       elsif (/^magnification$/) { $mag = $v; }
-      else { push(@unknown, [$op, $v]); } }
-    else { } }                                   # ?
-      # --------------------------------------------------
-      # Now, compile the options into a sequence of `transformations'.
-      # Note: the order of rotation & scaling is significant,
-      # but the order of various clipping options w.r.t rotation or scaling is not.
+      else                      { push(@unknown, [$op, $v]); } }
+    else { } }    # ?
+   # --------------------------------------------------
+   # Now, compile the options into a sequence of `transformations'.
+   # Note: the order of rotation & scaling is significant,
+   # but the order of various clipping options w.r.t rotation or scaling is not.
   my @transform = ();
   # We ignore viewport & trim if clip isn't set, since in that case we shouldn't
   # actually remove anything from the image (and there's no way to have the image
@@ -136,25 +158,26 @@ sub image_graphicx_parse {
   push(@transform, [($trim ? 'trim' : 'clip'), @vp]) if (@vp && $clip);
   push(@transform, ['rotate', $angle]) if ($rotfirst && $angle);    # Rotate before scaling?
   if ($width && $height) { push(@transform, ['scale-to', $mag * $width, $mag * $height, $aspect]); }
-  elsif ($width)  { push(@transform, ['scale-to', $mag * $width, 999999,         1]); }
-  elsif ($height) { push(@transform, ['scale-to', 999999,        $mag * $height, 1]); }
+  elsif ($width)             { push(@transform, ['scale-to', $mag * $width, 999999, 1]); }
+  elsif ($height)            { push(@transform, ['scale-to', 999999, $mag * $height, 1]); }
   elsif ($xscale && $yscale) { push(@transform, ['scale', $mag * $xscale, $mag * $yscale]); }
-  elsif ($xscale)   { push(@transform, ['scale', $mag * $xscale, $mag]); }
-  elsif ($yscale)   { push(@transform, ['scale', $mag,           $mag * $yscale]); }
-  elsif ($mag != 1) { push(@transform, ['scale', $mag,           $mag]); }
+  elsif ($xscale)            { push(@transform, ['scale', $mag * $xscale, $mag]); }
+  elsif ($yscale)            { push(@transform, ['scale', $mag, $mag * $yscale]); }
+  elsif ($mag != 1)          { push(@transform, ['scale', $mag, $mag]); }
   push(@transform, ['rotate', $angle]) if (!$rotfirst && $angle);    # Rotate after scaling?
                                                                      #  ----------------------
   return [@transform, @unknown]; }
 
 my %BP_conversions = (                                               # CONSTANT
   pt => 72 / 72.27, pc => 12 / 72.27, in => 72, bp => 1,
-  cm => 72 / 2.54, mm => 72 / 25.4, dd => (72 / 72.27) * (1238 / 1157),
+  cm => 72 / 2.54,  mm => 72 / 25.4,  dd => (72 / 72.27) * (1238 / 1157),
   cc => 12 * (72 / 72.27) * (1238 / 1157), sp => 72 / 72.27 / 65536);
 
 sub to_bp {
   my ($x) = @_;
   if ($x =~ /^\s*([+-]?[\d\.]+)(\w*)\s*$/) {
     my ($v, $u) = ($1, $2);
+    $u =~ s/^true//;
     return $v * ($u ? $BP_conversions{$u} : 1); }
   else {
     return 1; } }
@@ -175,7 +198,7 @@ sub image_graphicx_size {
       # $a1 => width (pts), $a2 => height (pts), $a3 => preserve aspect ratio.
       if ($a3) {             # If keeping aspect ratio, ignore the most extreme request
         if ($a1 / $w < $a2 / $h) { $a2 = $h * $a1 / $w; }
-        else { $a1 = $w * $a2 / $h; } }
+        else                     { $a1 = $w * $a2 / $h; } }
       ($w, $h) = (ceil($a1 * $dppt), ceil($a2 * $dppt)); }
     elsif ($op eq 'rotate') {
       my $rad = -$a1 * 3.1415926 / 180;    # close enough
@@ -194,20 +217,22 @@ sub image_graphicx_size {
         ($x0, $y0, $ww, $hh) = (floor($a1 * $dppt), floor($h - $a4 * $dppt),
           ceil(($a3 - $a1) * $dppt), ceil(($a4 - $a2) * $dppt)); }
       ($w, $h) = ($ww, $hh);
-    } }
+  } }
   return ($w, $h); }
 
 # Totally doesn't belong here, but want to share...
 sub image_graphicx_sizer {
   my ($whatsit) = @_;
   if (my $candidates = $whatsit->getProperty('candidates')) {
-    my ($source) = split(/,/, $candidates);
-    if (!pathname_is_absolute($source)) {
-      if (my $base = $STATE->lookupValue('SOURCEDIRECTORY')) {
-        $source = pathname_concat($base, $source); } }
-    my $options = $whatsit->getProperty('options');
-    my ($w, $h) = image_graphicx_size($source, image_graphicx_parse($options));
-    return (Dimension($w / $DOTS_PER_POINT . 'pt'), Dimension($h / $DOTS_PER_POINT . 'pt'), Dimension(0)) if $w; }
+    foreach my $source (split(/,/, $candidates)) {
+      if (!pathname_is_absolute($source)) {
+        if (my $base = $STATE->lookupValue('SOURCEDIRECTORY')) {
+          $source = pathname_concat($base, $source); } }
+      # Skip anything that a lower level imgsize can't understand
+      my $options = $whatsit->getProperty('options');
+      local $LaTeXML::IGNORE_ERRORS = 1;
+      my ($w, $h) = image_graphicx_size($source, image_graphicx_parse($options));
+      return (Dimension($w / $DOTS_PER_POINT . 'pt'), Dimension($h / $DOTS_PER_POINT . 'pt'), Dimension(0)) if $w; } }
   return (Dimension(0), Dimension(0), Dimension(0)); }
 
 #======================================================================
@@ -240,7 +265,7 @@ sub image_graphicx_trivial {
     elsif ($op eq 'scale-to') {    # $a1 => width, $a2 => height, $a3 => preserve aspect ratio.
       if ($a3) {                   # If keeping aspect ratio, ignore the most extreme request
         if ($a1 / $w < $a2 / $h) { $a2 = $h * $a1 / $w; }
-        else { $a1 = $w * $a2 / $h; } }
+        else                     { $a1 = $w * $a2 / $h; } }
       ($w, $h) = (ceil($a1 * $dppt), ceil($a2 * $dppt)); } }
   return ($w, $h); }
 
@@ -251,7 +276,14 @@ sub image_graphicx_complex {
   my ($source, $transform, %properties) = @_;
   my $dppt       = $properties{dppt}       || $DOTS_PER_POINT;
   my $background = $properties{background} || $BACKGROUND;
-  my $image = image_read($source, antialias => 1) or return;
+  my $image      = image_read($source, antialias => 1) or return;
+  # Get some defaults from the read-in image.
+  my ($imagedpi) = image_getvalue($image, 'x-resolution');
+  # image_setvalue($image,debug=>'exception');
+  my ($bg) = image_getvalue($image, 'transparent-color');
+  $background = "rgba($bg)" if $bg;    # Use background from image, if any.
+  my ($hasalpha) = image_getvalue($image, 'matte');
+
   image_internalop($image, 'Trim') or return if $properties{autocrop};
   my $orig_ncolors = image_getvalue($image, 'colors');
   return unless $orig_ncolors;
@@ -276,14 +308,15 @@ sub image_graphicx_complex {
         # $a1 => width (pts), $a2 => height (pts), $a3 => preserve aspect ratio.
         if ($a3) {             # If keeping aspect ratio, ignore the most extreme request
           if ($a1 / $w < $a2 / $h) { $a2 = $h * $a1 / $w; }
-          else { $a1 = $w * $a2 / $h; } }
+          else                     { $a1 = $w * $a2 / $h; } }
         ($w, $h) = (ceil($a1 * $dppt), ceil($a2 * $dppt)); } }
     my $X = 4;                 # Expansion factor
     my ($dx, $dy) = (int($X * 72 * $w / $w0), int($X * 72 * $h / $h0));
-    NoteProgressDetailed(" [reloading to desired size $w x $h (density = $dx x $dy)]");
+    Debug("reloading $source to desired size $w x $h (density = $dx x $dy)")
+      if $LaTeXML::DEBUG{images};
     $image = image_read($source, antialias => 1, density => $dx . 'x' . $dy) or return;
     image_internalop($image, 'Trim') or return if $properties{autocrop};
-    image_setvalue($image, colorspace => 'RGB') or return;
+    #    image_setvalue($image, colorspace => 'RGB') or return;
     image_internalop($image, 'Scale', geometry => int(100 / $X) . "%") or return;    # Now downscale.
     ($w, $h) = image_getvalue($image, 'width', 'height');
     return unless $w && $h; }
@@ -291,20 +324,22 @@ sub image_graphicx_complex {
   my $notes = '';
   foreach my $trans (@transform) {
     my ($op, $a1, $a2, $a3, $a4) = @$trans;
+    return unless $w && $h;
     if ($op eq 'scale') {                                                            # $a1 => scale
       ($w, $h) = (ceil($w * $a1), ceil($h * ($a2 || $a1)));
+      return unless $w && $h;
       $notes .= " scale to $w x $h";
       image_internalop($image, 'Scale', width => $w, height => $h) or return; }
     elsif ($op eq 'scale-to') {
       # $a1 => width (pts), $a2 => height (pts), $a3 => preserve aspect ratio.
       if ($a3) {    # If keeping aspect ratio, ignore the most extreme request
         if ($a1 / $w < $a2 / $h) { $a2 = $h * $a1 / $w; }
-        else { $a1 = $w * $a2 / $h; } }
+        else                     { $a1 = $w * $a2 / $h; } }
       ($w, $h) = (ceil($a1 * $dppt), ceil($a2 * $dppt));
       $notes .= " scale-to $w x $h";
       image_internalop($image, 'Scale', width => $w, height => $h) or return; }
     elsif ($op eq 'rotate') {
-      image_internalop($image, 'Rotate', degrees => -$a1, color => $background) or return;
+      image_internalop($image, 'Rotate', degrees => -$a1, background => $background) or return;
       ($w, $h) = image_getvalue($image, 'width', 'height');
       return unless $w && $h;
       $notes .= " rotate $a1 to $w x $h"; }
@@ -315,13 +350,15 @@ sub image_graphicx_complex {
     # but ImageMagick's coordinates are relative to upper left.
     elsif (($op eq 'trim') || ($op eq 'clip')) {
       my ($x0, $y0, $ww, $hh);
+      # Use the image's dpi for trim & clip!
+      my $idppt = (defined $imagedpi ? ($imagedpi / 72.0) : $dppt);
       if ($op eq 'trim') {    # Amount to trim: a1=left, a2=bottom, a3=right, a4=top
-        ($x0, $y0, $ww, $hh) = (floor($a1 * $dppt), floor($a4 * $dppt),
-          ceil($w - ($a1 + $a3) * $dppt), ceil($h - ($a4 + $a2) * $dppt));
+        ($x0, $y0, $ww, $hh) = (floor($a1 * $idppt), floor($a4 * $idppt),
+          ceil($w - ($a1 + $a3) * $idppt), ceil($h - ($a4 + $a2) * $idppt));
         $notes .= " trim to $ww x $hh @ $x0,$y0"; }
       else {                  # BBox: a1=left, a2=bottom, a3=right, a4=top
-        ($x0, $y0, $ww, $hh) = (floor($a1 * $dppt), floor($h - $a4 * $dppt),
-          ceil(($a3 - $a1) * $dppt), ceil(($a4 - $a2) * $dppt));
+        ($x0, $y0, $ww, $hh) = (floor($a1 * $idppt), floor($h - $a4 * $idppt),
+          ceil(($a3 - $a1) * $idppt), ceil(($a4 - $a2) * $idppt));
         $notes .= " clip to $ww x $hh @ $x0,$y0"; }
 
       if (($x0 > 0) || ($y0 > 0) || ($x0 + $ww < $w) || ($y0 + $hh < $h)) {
@@ -333,12 +370,17 @@ sub image_graphicx_complex {
         $h = min($hh + $y0, $h - $y0p);
         $notes .= " crop $w x $h @ $x0p,$y0p"; }
       # No direct `padding' operation in ImageMagick
-      my $nimage = image_read("xc:$background", size => "$ww x $hh") or return;
-      image_internalop($nimage, 'Composite', image => $image, compose => 'over', x => -$x0, y => -$y0) or return;
-      $image = $nimage;
-      ($w, $h) = ($ww, $hh);
-    } }
-  if (my $trans = $properties{transparent}) {
+      # BUT, trim & clip really shouldn't need to pad???
+      # And besides, this composition seems to mangle the colormap (depending on background)
+      # my $nimage = image_read("xc:$background", size => "$ww x $hh") or return;
+      # image_internalop($nimage, 'Composite', image => $image, compose => 'over',
+      #                  x => -$x0, y => -$y0) or return;
+      # $notes .= " compose to $ww x $hh at $x0,$y0";
+      # $image = $nimage;
+      # ($w, $h) = ($ww, $hh);
+  } }
+
+  if ($properties{transparent} && !$hasalpha) {
     $notes .= " transparent=$background";
     image_internalop($image, 'Transparent', $background) or return; }
 
@@ -353,7 +395,7 @@ sub image_graphicx_complex {
     $notes .= " quality=$quality";
     image_setvalue($image, quality => $properties{quality}) or return; }
 
-  NoteProgressDetailed(" [Transformed : $notes]") if $notes;
+  Debug("Transformed $source : $notes") if $notes && $LaTeXML::DEBUG{images};
   return ($image, $w, $h); }
 
 # Wrap up ImageMagick's methods to give more useful & consistent error handling.
@@ -396,6 +438,11 @@ sub image_setvalue {
     Error('imageprocessing', 'setvalue', undef, "No image object!"); return; }
   return image_internalop($image, 'Set', @args); }
 
+# Apparently ignorable warnings from Image::Magick
+our %ignorable = map { $_ => 1; } (
+  350,    # profile 'icc' not permitted on grayscale PNG
+);
+
 sub image_internalop {
   my ($image, $operation, @args) = @_;
   if (!$image) {
@@ -407,7 +454,8 @@ sub image_internalop {
     $retcode = $1; }
   if ($retcode < 400) {    # Warning
     Warn('imageprocessing', $operation, undef,
-      "Image processing operation $operation (" . join(', ', @args) . ") returned $retval");
+      "Image processing operation $operation (" . join(', ', @args) . ") returned $retval")
+      unless $ignorable{$retcode};
     return 1; }
   else {                   # Error
     Error('imageprocessing', $operation, undef,
